local SM = AceLibrary("SharedMedia-1.0")
local Table={}
Relog.Table=Table

local Columns={}
function Columns:SetupDivider(Divider)
	Divider:SetMovable(true)
	Divider:EnableMouse(true)
	
	Divider:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
		
	Divider:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
							this:StopMovingOrSizing();
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)							
							this.isMoving = false;

							if this.Left then
								this.Left:FixText()
							end

							if this.Right then
								this.Right:FixText()
							end
						end
						end)
	Divider:SetScript("OnHide", function() 
						if ( this.isMoving ) then
							this:StopMovingOrSizing();
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)
							this.isMoving = false;

							if this.Left then
								this.Left:FixText()
							end

							if this.Right then
								this.Right:FixText()
							end
						 end
						end)
	Divider:SetScript("OnUpdate", function() 
						if ( this.isMoving ) then
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)

							if this.Left then
								this.Left:FixText()
							end

							if this.Right then
								this.Right:FixText()
							end
						end
						end)	
end

function Columns:SetRowNum(num)
	if num>#self.Rows then
		for i=(#self.Rows+1),num do
			self.Rows[i]=self:CreateFontString(nil,"OVERLAY","GameFontNormal")
			self.Rows[i]:SetTextColor(1.0,1.0,1.0)			
		end
	elseif num<#self.Rows then
		for i=num,#self.Rows do
			self.Rows[i]:Hide()
		end
	end

	for i=1,num do
		self.Rows[i]:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)
		self.Rows[i]:SetPoint("TOP",self,"TOP",0,-i*(1+Relog.db.profile.RowHeight))
		self.Rows[i]:Show()
	end
end

function Columns:SetRow(Row,Table)
	self.Rows[Row]:SetText(Relog.Tags.CompileTag(self.Tag,Table))

	local MaxWidth
	if self:GetRight() and self:GetLeft() then
		MaxWidth=self:GetRight() - self:GetLeft()
	else
		MaxWidth=self:GetWidth() 
	end
	local Text=self.Rows[Row]:GetText()

	self.Rows[Row].Text=Text

	while self.Rows[Row]:GetStringWidth()>MaxWidth do
		Text=strsub(Text,1,#Text-1)
		self.Rows[Row]:SetText(Text.."...")
		if Text=="" then
			break
		end
	end
end

function Columns:FixText()
	local Text
	local MaxWidth
	if self:GetRight() and self:GetLeft() then
		MaxWidth=self:GetRight() - self:GetLeft()
	else
		MaxWidth=self:GetWidth() 
	end
	for _,Row in pairs(self.Rows) do
		Text=Row.Text
		Row:SetText(Text)
		while Row:GetStringWidth()>MaxWidth do
			Text=strsub(Text,1,#Text-1)
			Row:SetText(Text.."...")
			if Text=="" then
				break
			end
		end
	end
end




function Table:AddColumn(Name,Tag)
	local Column=CreateFrame("FRAME",nil,self)
	Column:SetWidth(100)
	Column:SetHeight(self:GetHeight()-2)

	Column.Tag=Tag
	Column.Rows={}
	Column.SetRowNum=Columns.SetRowNum
	Column.SetRow=Columns.SetRow
	Column.FixText=Columns.FixText

	Column.Title=CreateFrame("FRAME",nil,Column)
	Column.Title:SetHeight(Relog.db.profile.RowHeight)
	Column.Title:SetPoint("TOPLEFT",Column,"TOPLEFT")
	Column.Title:SetPoint("TOPRIGHT",Column,"TOPRIGHT")

	Column.Title.Text=Column.Title:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Column.Title.Text:SetTextColor(1.0,1.0,1.0)
	Column.Title.Text:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)
	Column.Title.Text:SetPoint("CENTER",Column.Title,"CENTER")
	Column.Title.Text:SetText(Name)

	Column.Title.Background=Column.Title:CreateTexture(nil,"BACKGROUND")
	Column.Title.Background:SetAllPoints(Column.Title)

	if math.fmod(#self.Columns,2)==0 then
		Column.Title.Background:SetTexture(0,0,0,0.4)
	else
		Column.Title.Background:SetTexture(0.3,0.3,0.3,0.4)
	end

	if #self.Columns==0 then
		Column:SetPoint("TOPLEFT",self,"TOPLEFT",1,-1)
	else
		local Divider=CreateFrame("FRAME",nil,self)
		Divider:SetWidth(7)
		Divider:SetHeight(self:GetHeight()-2)
		Divider:ClearAllPoints()
		Divider:SetPoint("CENTER",self,"LEFT",10,0)
		Divider:SetFrameLevel(Divider:GetFrameLevel()+1)

		Divider.Background=Divider:CreateTexture(nil,"OVERLAY")
		Divider.Background:SetWidth(1)
		Divider.Background:SetPoint("TOP",Divider,"TOP")
		Divider.Background:SetPoint("BOTTOM",Divider,"BOTTOM")
		Divider.Background:SetTexture(1.0,1.0,1.0)
		Columns:SetupDivider(Divider)

		self.Dividers[#self.Dividers+1]=Divider
		
		Divider.Left=self.Columns[#self.Columns]
		Divider.Right=Column
		self.Columns[#self.Columns]:SetPoint("BOTTOMRIGHT",Divider,"BOTTOM")
		Column:SetPoint("TOPLEFT",Divider,"TOP")
	end
	Column:SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-1,1)

	self.Columns[#self.Columns+1]=Column
	self:UpdateDividers()
end

function Table:UpdateDividers()
	local DividerNum=#self.Dividers
	local Spacing=(self:GetWidth()-2)/(DividerNum+1)

	for k, v in pairs(self.Dividers) do
		v:SetPoint("CENTER",self,"LEFT",1+k*Spacing,0)
	end
end

function Table:UpdateRows()
	local RowNum=self:GetHeight()/(1+Relog.db.profile.RowHeight)-1

	for k,v in pairs(self.Columns) do
		v:SetRowNum(RowNum)
	end

	for i=1,RowNum do
		local log=Relog:GetLog(i)
		for k,v in pairs(self.Columns) do
			v:SetRow(i,log)
		end
	end
end

function Table:CreateTable()
	local theFrame=CreateFrame("FRAME",nil,UIParent)

	theFrame:ClearAllPoints()
	theFrame:SetPoint("CENTER",UIParent)
	theFrame:SetHeight(172)
	theFrame:SetWidth(400)

	theFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeSize = 1,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
	})
	theFrame:SetBackdropBorderColor(1.0,1.0,1.0)
	theFrame:SetBackdropColor(24/255, 24/255, 24/255)
	
	theFrame:EnableMouse(true)
	theFrame:SetMovable(true)
	theFrame:SetResizable(true)
	
	theFrame:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
	theFrame:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)
	theFrame:SetScript("OnHide", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)	
	theFrame.Columns={}
	theFrame.Dividers={}
	theFrame.AddColumn=Table.AddColumn
	theFrame.UpdateDividers=Table.UpdateDividers
	theFrame.UpdateRows=Table.UpdateRows

	return theFrame
end

function Relog:CreateTestTable()
	local a=Relog.Table:CreateTable()
	a:AddColumn("Time","[Date(\"%H:%M:%S.%s\",Time)]")
	a:AddColumn("Type","[Type]")
	a:AddColumn("Source","[Source]")
	a:AddColumn("Victim","[Victim]")
	a:AddColumn("Amount","[Amount]");
	a:UpdateRows()
end