local SM = AceLibrary("SharedMedia-1.0")
local Window={}
Relog.Window=Window

function Window:CreateRow(num)
	local Row=CreateFrame("Frame", nil,Window.theWindow)

	Row:SetHeight(Relog.db.profile.RowHeight)
	Row:SetWidth(Window.theWindow:GetWidth()-4-16)

	Row:SetPoint("TOP",Window.theWindow,"TOP",-8,-2-(Relog.db.profile.RowHeight+1)*(num-1))

	Row.Text=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Row.Text:SetTextColor(1.0,1.0,1.0)
	Row.Text:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)
	Row.Text:SetPoint("LEFT",Row,"LEFT")

	Window.Rows[num]=Row
end

function Window:ResizeWindow()
	local RowNum=math.floor((Window.theWindow:GetHeight()-2)/(Relog.db.profile.RowHeight+1))
	
	for k, v in pairs(Window.Rows) do
		v:Hide()
	end

	for i=1,RowNum do
		if Window.Rows[i] then
			Window.Rows[i]:Show()
			Window.Rows[i]:SetPoint("TOP",Window.theWindow,"TOP",-8,-2-(Relog.db.profile.RowHeight+1)*(i-1))
		else
			Window:CreateRow(i)
		end
	end
end

function Window:RefreshRows()
	local RowNum=math.floor((Window.theWindow:GetHeight()-2)/(Relog.db.profile.RowHeight+1))
	local offset

	if Window.ScrollBar then
		FauxScrollFrame_Update(Window.ScrollBar, Relog.db.char.LogSize, RowNum, Relog.db.profile.RowHeight)
		offset = FauxScrollFrame_GetOffset(Window.ScrollBar)
		Relog.db.char.LogOffset=offset
	else
		offset = 0
	end

	local MsgNum=Relog.db.char.StartLog+offset

	Window.RowNum=RowNum

	if MsgNum>Relog.db.profile.MaxLogs then
		MsgNum=MsgNum-Relog.db.profile.MaxLogs
	end

	if MsgNum<1 then
		MsgNum=MsgNum+Relog.db.profile.MaxLogs
	end

	for i=1,RowNum do
		if Relog.db.char.Log.CompiledCache[MsgNum] then
			Window.Rows[i].Text:SetText(Relog.db.char.Log.CompiledCache[MsgNum])
		else
			Window.Rows[i].Text:SetText("")
		end
		MsgNum=MsgNum+1
		if MsgNum>Relog.db.profile.MaxLogs then
			MsgNum=MsgNum-Relog.db.profile.MaxLogs
		end
	end
end

function Window:CreateWindow()
	local theFrame=CreateFrame("Frame", "Relow_LogWindow",UIParent)
	
	Window.theWindow=theFrame
	
	theFrame:ClearAllPoints()
	theFrame:SetPoint("CENTER",UIParent)
	theFrame:SetHeight(172)
	theFrame:SetWidth(400)

	theFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeSize = 1,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
	})
	theFrame:SetBackdropBorderColor(1.0,1.0,1.0)
	theFrame:SetBackdropColor(24/255, 24/255, 24/255)
	
	theFrame:EnableMouse(true)
	theFrame:SetMovable(true)
	theFrame:SetResizable(true)
	
	theFrame:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
	theFrame:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)
	theFrame:SetScript("OnHide", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)
	Window.Rows={}
	Window:ResizeWindow()
	Window:RefreshRows()	

	

	Window.ScrollBar=CreateFrame("SCROLLFRAME","Relog_Log_ScrollBar",theFrame,"FauxScrollFrameTemplate")
	Window.ScrollBar:SetScript("OnVerticalScroll", function() FauxScrollFrame_OnVerticalScroll(12, Window.RefreshRows) end)
	Window.ScrollBar:SetPoint("TOPLEFT",Window.Rows[1],"TOPLEFT")	
	Window.ScrollBar:SetPoint("BOTTOMRIGHT",Window.Rows[Window.RowNum],"BOTTOMRIGHT",-5,0)
	local Saved=Relog.db.char.LogOffset
	
	Window:RefreshRows()
	Relog.db.char.LogOffset=Saved
	Relog.Window.SetPos(Relog.db.char.LogOffset)
	

	--Relog:SetupScrollbar("Relog_Log_ScrollBar")
end

function Window.SetPos(offset)
	local scrollbar = getglobal("Relog_Log_ScrollBarScrollBar")
	

	local RowNum=math.floor((Window.theWindow:GetHeight()-2)/(Relog.db.profile.RowHeight+1))
	if offset==nil then
		offset=0
	end

	if offset>(Relog.db.char.LogSize-RowNum) then
		offset=math.max(0,Relog.db.char.LogSize-RowNum)
	end

	FauxScrollFrame_Update(Window.ScrollBar, Relog.db.char.LogSize, RowNum, Relog.db.profile.RowHeight)

	local min, max = scrollbar:GetMinMaxValues()

	scrollbar:SetValue(max*offset/(Relog.db.char.LogSize-Window.RowNum)+min)
	Window.ScrollBar.offset = offset
	Window:RefreshRows()
end

function Relog:SetupScrollbar(name)
	local Thumb=getglobal(name.."ScrollBarThumbTexture")
	Thumb:SetTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-Knob")
	Thumb:SetVertexColor(0.75,0.75,0.75)

	local Up=getglobal(name.."ScrollBarScrollUpButton")
	Up:SetNormalTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollUpButton-Up")
	Up:SetPushedTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollUpButton-Up")
	Up:SetDisabledTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollUpButton-Disabled")
	Up:SetHighlightTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollUpButton-Highlight")

	if not Up.Overlay then
		Up.Overlay=Up:CreateTexture(nil,"OVERLAY")
		Up.Overlay:SetAllPoints(Up)
		Up.Overlay:SetTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollUpButton-Overlay")
		Up.Overlay:SetVertexColor(0.75,0.75,0.75)
		Up.Overlay:SetTexCoord(0.25,0.75,0.25,0.75)
		Up.Overlay:SetBlendMode("MOD")
	end

	local Down=getglobal(name.."ScrollBarScrollDownButton")
	Down:SetNormalTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollDownButton-Up")
	Down:SetPushedTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollDownButton-Up")
	Down:SetDisabledTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollDownButton-Disabled")
	Down:SetHighlightTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollDownButton-Highlight")

	if not Down.Overlay then
		Down.Overlay=Up:CreateTexture(nil,"OVERLAY")
		Down.Overlay:SetAllPoints(Down)
		Down.Overlay:SetTexture("Interface\\AddOns\\Recount\\textures\\scrollbar\\UI-ScrollBar-ScrollDownButton-Overlay")
		Down.Overlay:SetVertexColor(0.75,0.75,0.75)
		Down.Overlay:SetTexCoord(0.25,0.75,0.25,0.75)
		Down.Overlay:SetBlendMode("MOD")
	end
end